@@ -3,6 +3,7 @@
 from dataclasses import dataclass, fields
 from datetime import date, datetime
 from enum import Enum
+from typing import TypedDict
 
 import aiohttp
 import pytest
@@ -15,6 +16,10 @@ from pypaperless.exceptions import (
     DraftNotSupportedError,
     InitializationError,
     JsonResponseWithError,
+    PaperlessConnectionError,
+    PaperlessForbiddenError,
+    PaperlessInactiveOrDeletedError,
+    PaperlessInvalidTokenError,
 )
 from pypaperless.models import Page
 from pypaperless.models.base import HelperBase, PaperlessModel
@@ -60,6 +65,11 @@ class TestPaperless:
 
     async def test_context(self, resp: aioresponses, api: Paperless) -> None:
         """Test context."""
+        resp.get(
+            f"{PAPERLESS_TEST_URL}{API_PATH['api_schema']}",
+            status=500,
+            payload=PATCHWORK["paths"],
+        )
         resp.get(
             f"{PAPERLESS_TEST_URL}{API_PATH['index']}",
             status=200,
@@ -90,16 +100,58 @@ class TestPaperless:
 
     async def test_init_error(self, resp: aioresponses, api: Paperless) -> None:
         """Test initialization error."""
+        # simulate connection due no configuration error
+        with pytest.raises(PaperlessConnectionError):
+            await api.initialize()
+
         # http status error
+        resp.get(
+            f"{PAPERLESS_TEST_URL}{API_PATH['api_schema']}",
+            status=500,
+            payload=PATCHWORK["paths"],
+        )
         resp.get(
             f"{PAPERLESS_TEST_URL}{API_PATH['index']}",
             status=401,
             body="any html",
         )
-        with pytest.raises(InitializationError):
+        with pytest.raises(PaperlessInvalidTokenError):
+            await api.initialize()
+
+        # http 401 - inactive or deleted user
+        resp.get(
+            f"{PAPERLESS_TEST_URL}{API_PATH['api_schema']}",
+            status=500,
+            payload=PATCHWORK["paths"],
+        )
+        resp.get(
+            f"{PAPERLESS_TEST_URL}{API_PATH['index']}",
+            status=401,
+            payload={"detail": "User is inactive"},
+        )
+        with pytest.raises(PaperlessInactiveOrDeletedError):
+            await api.initialize()
+
+        # http status forbidden
+        resp.get(
+            f"{PAPERLESS_TEST_URL}{API_PATH['api_schema']}",
+            status=500,
+            payload=PATCHWORK["paths"],
+        )
+        resp.get(
+            f"{PAPERLESS_TEST_URL}{API_PATH['index']}",
+            status=403,
+            body="any html",
+        )
+        with pytest.raises(PaperlessForbiddenError):
             await api.initialize()
 
         # http ok, wrong payload
+        resp.get(
+            f"{PAPERLESS_TEST_URL}{API_PATH['api_schema']}",
+            status=500,
+            payload=PATCHWORK["paths"],
+        )
         resp.get(
             f"{PAPERLESS_TEST_URL}{API_PATH['index']}",
             status=200,
@@ -108,6 +160,19 @@ class TestPaperless:
         with pytest.raises(InitializationError):
             await api.initialize()
 
+    @pytest.mark.parametrize(
+        "exception_cls",
+        [
+            PaperlessConnectionError,
+            PaperlessInvalidTokenError,
+            PaperlessInactiveOrDeletedError,
+            PaperlessForbiddenError,
+        ],
+    )
+    async def test_errors_are_backwards_compatible(self, exception_cls: type) -> None:
+        """Test, if new errors are backwards compatible."""
+        assert issubclass(exception_cls, InitializationError)
+
     async def test_jsonresponsewitherror(self) -> None:
         """Test JsonResponseWithError."""
         try:
@@ -338,7 +403,7 @@ class TestPaperless:
         test.extra_data = None
         assert test.label is None
 
-    async def test_dataclass_conversion(self) -> None:
+    async def test_dataclass_conversion(self) -> None:  # pylint: disable=too-many-statements
         """Test dataclass utils."""
 
         class SomeStatus(Enum):
@@ -353,6 +418,20 @@ class TestPaperless:
                 """Set default."""
                 return cls.UNKNOWN
 
+        class SomeNestedExtraData(TypedDict):
+            """Test nested TypedDict."""
+
+            ustr: str | None
+            uany: int | str | bool | None
+
+        class SomeExtraData(TypedDict):
+            """Test TypedDict."""
+
+            a_str: str
+            a_dict: dict[str, str]
+            a_list: list[str]
+            a_typeddict: SomeNestedExtraData
+
         @dataclass
         class SomeFriend:
             """Test class."""
@@ -360,6 +439,11 @@ class TestPaperless:
             name: str
             age: int
 
+            @classmethod
+            def from_dict(cls, data: dict) -> "SomeFriend":
+                """Test from_dict stuff."""
+                return cls(name=str(data.get("name")), age=int(data.get("age")))
+
         @dataclass
         class SomePerson:
             """Test class."""
@@ -367,6 +451,7 @@ class TestPaperless:
             name: str
             age: int
             height: float
+            height2: float
             birth: date
             last_login: datetime
             friends: list[SomeFriend] | None
@@ -375,11 +460,13 @@ class TestPaperless:
             status: SomeStatus
             file: bytes
             meta: dict[str, str]
+            extra_data: SomeExtraData
 
         raw_data = {
             "name": "Lee Tobi, Sajangnim",
             "age": 38,
             "height": 1.76,
+            "height2": 2,
             "birth": "1986-05-23",
             "last_login": "2023-08-08T06:06:35.495972Z",
             "is_deleted": False,
@@ -396,6 +483,15 @@ class TestPaperless:
             "status": 1,
             "file": b"5-23-42-666-0815-1337",
             "meta": {"hairs": "blonde", "eyes": "blue", "loves": "Python"},
+            "extra_data": {
+                "a_str": "test",
+                "a_dict": {
+                    "key1": "val1",
+                    "key2": "val2",
+                },
+                "a_list": ["a", "b", "c"],
+                "a_typeddict": {"ustr": "hello", "uany": 1},
+            },
         }
 
         data = {
@@ -412,6 +508,7 @@ class TestPaperless:
         assert isinstance(res.name, str)
         assert isinstance(res.age, int)
         assert isinstance(res.height, float)
+        assert isinstance(res.height2, float)
         assert isinstance(res.birth, date)
         assert isinstance(res.last_login, datetime)
         assert isinstance(res.friends, list)
@@ -422,12 +519,16 @@ class TestPaperless:
         assert res.is_deleted is False
         assert isinstance(res.status, SomeStatus)
         assert isinstance(res.file, bytes)
+        assert isinstance(res.extra_data, dict)
+        assert isinstance(res.extra_data["a_typeddict"], dict)
 
         # back conversion
         back = {field.name: object_to_dict_value(getattr(res, field.name)) for field in fields(res)}
 
         assert isinstance(back["friends"][0]["age"], int)  # was str in the source dict
         assert isinstance(back["meta"], dict)
+        assert isinstance(back["extra_data"], dict)
+        assert isinstance(back["extra_data"]["a_list"], list)
 
     async def test_pages_object(self, api: Paperless) -> None:
         """Test pages."""
